Class: Authgasm::Sha256CryptoProvider
- Inherits:
-
Object
- Object
- Authgasm::Sha256CryptoProvider
- Defined in:
- lib/authgasm/sha256_crypto_provider.rb
Overview
Sha256 Crypto Provider
The acts_as_authentic method allows you to pass a :crypto_provider option. This allows you to use any type of encryption you like. Just create a class with a class level encrypt and decrypt method. The password will be passed as the single parameter to each of these methods so you can do your magic.
If you are encrypting via a hash just don’t include a decrypt method, since hashes can’t be decrypted. Authgasm will notice this adjust accordingly.
Class Method Summary collapse
Class Method Details
.encrypt(pass) ⇒ Object
9 10 11 |
# File 'lib/authgasm/sha256_crypto_provider.rb', line 9 def self.encrypt(pass) Digest::SHA256.hexdigest(pass) end |